import uuid
import transaction
from pyramid.view import view_config
from barker.models import FoodTable, Overview
from barker.models.validation_exception import ValidationError


@view_config(
    request_method="POST",
    route_name="v1_tables_new",
    renderer="json",
    permission="Tables",
    trans=True,
)
def save(request):
    json = request.json_body

    name = json.get("name", "").strip()
    if name == "":
        raise ValidationError("Name cannot be blank")
    try:
        seats = int(json.get("seats", 0))
        if seats < 0:
            raise ValidationError("Seats must be an integer >= 0")
    except ValueError:
        raise ValidationError("Seats must be an integer >= 0")
    section = json.get("section", None)
    if section is None:
        raise ValidationError("please choose a section")
    section_id = uuid.UUID(section["id"])
    is_active = json.get("isActive", True)
    item = FoodTable(name, seats, section_id, is_active)
    request.dbsession.add(item)
    transaction.commit()
    return food_table_info(item.id, request.dbsession)


@view_config(
    request_method="PUT",
    route_name="v1_tables_id",
    renderer="json",
    permission="Tables",
    trans=True,
)
def update(request):
    item = (
        request.dbsession.query(FoodTable)
        .filter(FoodTable.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    json = request.json_body
    item.name = json["name"].strip()
    if item.name == "":
        raise ValidationError("Name cannot be blank")
    try:
        item.seats = int(json.get("seats", 0))
        if item.seats < 0:
            raise ValidationError("Seats must be an integer >= 0")
    except ValueError:
        raise ValidationError("Seats must be an integer >= 0")
    item.section_id = uuid.UUID(json["section"]["id"])
    item.is_active = json.get("isActive", True)
    transaction.commit()
    return food_table_info(item.id, request.dbsession)


@view_config(
    request_method="DELETE",
    route_name="v1_tables_id",
    renderer="json",
    permission="Tables",
    trans=True,
)
def delete(request):
    item = (
        request.dbsession.query(FoodTable)
        .filter(FoodTable.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    request.dbsession.delete(item)
    transaction.commit()
    return {None, request.dbsession}


@view_config(
    request_method="GET",
    route_name="v1_tables_id",
    renderer="json",
    permission="Authenticated",
)
def show_id(request):
    id_ = uuid.UUID(request.matchdict["id"])
    return food_table_info(id_, request.dbsession)


@view_config(
    request_method="GET", route_name="v1_tables_new", renderer="json", request_param="v"
)
def show_voucher(request):
    voucher_id = request.GET["v"]
    voucher = (
        request.dbsession.query(Overview)
        .filter(Overview.voucher_id == voucher_id)
        .first()
    )
    if voucher is not None:
        return food_table_info(voucher.food_table, request.dbsession)
    return {}


@view_config(
    request_method="GET",
    route_name="v1_tables_new",
    renderer="json",
    permission="Tables",
)
def show_blank(request):
    return food_table_info(None, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_tables_list",
    renderer="json",
    request_param="r",
    permission="Authenticated",
)
def show_running(request):
    list_ = request.dbsession.query(FoodTable).filter(FoodTable.is_active == True).order_by(FoodTable.sort_order).all()

    food_tables = []
    for item in list_:
        ft = {
            "id": item.id,
            "name": item.name,
            "seats": item.seats,
            "section": {"id": item.section_id, "name": item.section.name},
            "isActive": item.is_active,
            "sortOrder": item.sort_order
        }
        if item.status is not None:
            ft["status"] = item.status.status
            ft["voucherId"] = item.status.voucher_id
            ft["pax"] = item.status.voucher.pax
            ft["date"] = item.status.voucher.date.strftime("%d-%b-%Y %H:%M")
            ft["amount"] = item.status.voucher.amount
            if item.status.guest is not None:
                ft["guest"] = item.status.guest.customer.name
        food_tables.append(ft)
    return food_tables


@view_config(
    request_method="GET",
    route_name="v1_tables_list",
    renderer="json",
    permission="Authenticated",
)
def show_list(request):
    return [{
        "id": item.id,
        "name": item.name,
        "seats": item.seats,
        "section": {"id": item.section_id, "name": item.section.name},
        "isActive": item.is_active,
        "sortOrder": item.sort_order
    } for item in request.dbsession.query(FoodTable).order_by(FoodTable.sort_order).all()]


@view_config(
    request_method="POST",
    route_name="v1_tables_list",
    renderer="json",
    permission="Tables",
)
def sort_order(request):
    json = request.json_body
    for index, item in enumerate(json):
        request.dbsession.query(FoodTable).filter(
            FoodTable.id == uuid.UUID(item["id"])
        ).update({FoodTable.sort_order: index})
    return True


def food_table_info(item, dbsession):
    if item is None:
        return {"name": "", "seats": 0, "section": {}, "isActive": True, "sortOrder": 0}
    if type(item) == uuid.UUID:
        item = dbsession.query(FoodTable).filter(FoodTable.id == item).first()
    return {
        "id": item.id,
        "name": item.name,
        "seats": item.seats,
        "section": {"id": item.section_id, "name":item.section.name},
        "isActive": item.is_active,
        "status": "" if item.status is None else item.status.status,
        "sortOrder": item.sort_order,
        "voucherId": None if item.status is None else item.status.voucher_id,
    }
