import uuid
from decimal import Decimal, InvalidOperation

import transaction
from pyramid.response import Response
from pyramid.view import view_config

from barker.models import Product
from barker.models.validation_exception import ValidationError


@view_config(
    request_method="POST",
    route_name="v1_products_new",
    renderer="json",
    permission="Products",
    trans=True,
)
def save(request):
    json = request.json_body

    name = json.get("name", "").strip()
    if name == "":
        raise ValidationError("Name cannot be blank")

    units = json.get("units", "").strip()

    menu_category = json.get("menuCategory", None)
    if menu_category is None:
        raise ValidationError("please choose a menu category")
    menu_category_id = uuid.UUID(menu_category["id"])

    sale_category = json.get("saleCategory", None)
    if sale_category is None:
        raise ValidationError("please choose a sale category")
    sale_category_id = uuid.UUID(sale_category["id"])

    try:
        price = Decimal(json.get("price", 0))
        if price < 0:
            raise ValidationError("Price must be a decimal >= 0")
    except (ValueError, InvalidOperation):
        raise ValidationError("Price must be a decimal >= 0")

    has_happy_hour = json.get("hasHappyHour", True)
    is_not_available = json.get("isNotAvailable", True)

    try:
        quantity = Decimal(json.get("quantity", 0))
        if quantity < 0:
            raise ValidationError("Quantity must be a decimal >= 0")
    except (ValueError, InvalidOperation):
        raise ValidationError("Quantity must be a decimal >= 0")

    is_active = json.get("isActive", True)

    item = Product(
        name,
        units,
        menu_category_id,
        sale_category_id,
        price,
        has_happy_hour,
        is_not_available,
        quantity,
        is_active,
    )
    request.dbsession.add(item)
    transaction.commit()
    return product_info(item.id, request.dbsession)


@view_config(
    request_method="PUT",
    route_name="v1_products_id",
    renderer="json",
    permission="Products",
    trans=True,
)
def update(request):
    item = (
        request.dbsession.query(Product)
        .filter(Product.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    json = request.json_body
    item.name = json["name"].strip()
    if item.name == "":
        raise ValidationError("Name cannot be blank")
    item.units = json["units"].strip()
    item.menu_category_id = uuid.UUID(json["menuCategory"]["id"])
    item.sale_category_id = uuid.UUID(json["saleCategory"]["id"])
    try:
        item.price = Decimal(json["price"])
        if item.price < 0:
            raise ValidationError("Price must be a decimal >= 0")
    except (ValueError, InvalidOperation):
        raise ValidationError("Price must be a decimal >= 0")
    item.has_happy_hour = json["hasHappyHour"]
    item.is_not_available = json["isNotAvailable"]
    try:
        item.quantity = Decimal(json["quantity"])
        if item.price < 0:
            raise ValidationError("Quantity must be a decimal >= 0")
    except (ValueError, InvalidOperation):
        raise ValidationError("Quantity must be a decimal >= 0")
    item.is_active = json["isActive"]
    transaction.commit()
    return product_info(item.id, request.dbsession)


@view_config(
    request_method="DELETE",
    route_name="v1_products_id",
    renderer="json",
    permission="Products",
    trans=True,
)
def delete(request):
    item = (
        request.dbsession.query(Product)
        .filter(Product.id == uuid.UUID(request.matchdict["id"]))
        .first()
    )
    if item is None:
        response = Response("Product not Found")
        response.status_int = 500
        return response
    else:
        response = Response("Product deletion not implemented")
        response.status_int = 500
        return response


@view_config(
    request_method="GET",
    route_name="v1_products_new",
    renderer="json",
    permission="Authenticated",
)
def show_blank(request):
    return product_info(None, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_products_id",
    renderer="json",
    permission="Authenticated",
)
def show_id(request):
    id_ = uuid.UUID(request.matchdict["id"])
    return product_info(id_, request.dbsession)


@view_config(
    request_method="GET",
    route_name="v1_products_list",
    renderer="json",
    permission="Authenticated",
)
def show_list(request):
    active = request.GET.get("a", None)
    list_ = request.dbsession.query(Product)
    if active is not None:
        list_ = list_.filter(Product.is_active == active)
    list_ = (
        list_.order_by(Product.menu_category_id, Product.sort_order)
        .order_by(Product.name)
        .all()
    )
    products = []
    for item in list_:
        products.append(product_info(item, request.dbsession))
    return products


@view_config(
    request_method="GET",
    route_name="v1_products_list",
    renderer="json",
    request_param=("mc", "a"),
    permission="Authenticated",
)
def show_list_sale(request):
    mc_id = uuid.UUID(request.GET["mc"])
    list_ = (
        request.dbsession.query(Product)
        .filter(Product.is_active == True, Product.menu_category_id == mc_id)
        .order_by(Product.sort_order, Product.name)
        .all()
    )
    products = []
    for item in list_:
        products.append(
            {
                "id": item.id,
                "name": item.name,
                "units": item.units,
                "saleCategory": {
                    "id": item.sale_category_id,
                    "name": item.sale_category.name,
                },
                "tax": {
                    "id": item.sale_category.tax_id,
                    "name": item.sale_category.tax.name,
                    "rate": item.sale_category.tax.rate,
                },
                "price": item.price,
                "hasHappyHour": False,
                "isNotAvailable": item.is_not_available,
                "isActive": item.is_active,
                "sortOrder": item.sort_order,
            }
        )
        if item.has_happy_hour:
            products.append(
                {
                    "id": item.id,
                    "name": "H H " + item.name,
                    "units": item.units,
                    "saleCategory": {
                        "id": item.sale_category_id,
                        "name": item.sale_category.name,
                    },
                    "tax": {
                        "id": item.sale_category.tax_id,
                        "name": item.sale_category.tax.name,
                        "rate": item.sale_category.tax.rate,
                    },
                    "price": item.price,
                    "hasHappyHour": True,
                    "isNotAvailable": item.is_not_available,
                    "isActive": item.is_active,
                    "sortOrder": item.sort_order,
                }
            )
    return products


@view_config(
    request_method="POST",
    route_name="v1_products_list",
    renderer="json",
    permission="Products",
    trans=True,
)
def sort_order(request):
    json = request.json_body
    indexes = {}
    for item in json:
        product_id = uuid.UUID(item["id"])
        menu_category_id = uuid.UUID(item["menuCategory"]["id"])
        if menu_category_id in indexes:
            indexes[menu_category_id] += 1
        else:
            indexes[menu_category_id] = 0
        request.dbsession.query(Product).filter(Product.id == product_id).update(
            {Product.sort_order: indexes[menu_category_id]}
        )
    return True


def product_info(item, dbsession):
    if item is None:
        return {
            "name": "",
            "units": "",
            "menuCategory": {},
            "saleCategory": {},
            "price": 0,
            "hasHappyHour": False,
            "isNotAvailable": False,
            "isActive": True,
            "sortOrder": 0,
        }
    if type(item) is uuid.UUID:
        item = dbsession.query(Product).filter(Product.id == item).first()
    return {
        "id": item.id,
        "name": item.name,
        "units": item.units,
        "menuCategory": {"id": item.menu_category_id, "name": item.menu_category.name},
        "saleCategory": {
            "id": item.sale_category_id,
            "name": item.sale_category.name,
            "tax": {
                "id": item.sale_category.tax_id,
                "name": item.sale_category.tax.name,
            },
        },
        "price": item.price,
        "hasHappyHour": item.has_happy_hour,
        "isNotAvailable": item.is_not_available,
        "isActive": item.is_active,
        "sortOrder": item.sort_order,
    }
