from datetime import datetime, timedelta
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import (
    Inventory,
    Kot,
    Product,
    Settlement,
    SettleOption,
    Tax,
    Voucher,
    SaleCategory,
    VoucherType,
)
from barker.models.validation_exception import ValidationError
from barker.views.reports import get_start_date, get_finish_date


@view_config(
    request_method="GET",
    route_name="v1_sale_analysis",
    renderer="json",
    permission="Sales Analysis",
)
def get_sale_analysis(request):
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    if (
        datetime.today() - start_date.replace(hour=0)
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "amounts": (
            get_sale(start_date, finish_date, request.dbsession)
            + [{"name": "--", "amount": 0}]
            + get_settlements(start_date, finish_date, request.dbsession)
            + [{"name": "--", "amount": 0}]
            + get_tax(start_date, finish_date, request.dbsession)
        ),
    }


@view_config(
    request_method="GET",
    route_name="v1_sa_sale",
    renderer="json",
    permission="Sales Analysis",
)
def get_sale_view(request):
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    if (
        datetime.today() - start_date.replace(hour=0)
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "amounts": get_sale(start_date, finish_date, request.dbsession),
    }


def get_sale(start_date, finish_date, dbsession):
    list_ = (
        dbsession.query(SaleCategory.name, func.sum(Inventory.net))
        .join(Inventory.kot)
        .join(Kot.voucher)
        .join(Inventory.product)
        .join(Product.sale_category)
        .filter(
            Voucher.date >= start_date,
            Voucher.date <= finish_date,
            Voucher.voucher_type == VoucherType.REGULAR_BILL.value,
        )
        .group_by(SaleCategory.name)
        .order_by(SaleCategory.name)
        .all()
    )
    total = 0
    info = []
    for gt, am in list_:
        total += am
        info.append({"name": gt, "amount": am})
    return info + [{"name": "Total Settled", "amount": total}]


@view_config(
    request_method="GET",
    route_name="v1_sa_settlements",
    renderer="json",
    permission="Sales Analysis",
)
def get_settlements_view(request):
    start_date = get_start_date(request.GET.get("s", None))
    finish_date = get_finish_date(request.GET.get("f", None))

    if (
        datetime.today() - start_date.replace(hour=0)
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "amounts": get_settlements(start_date, finish_date, request.dbsession),
    }


def get_settlements(start_date, finish_date, dbsession):
    list_ = (
        dbsession.query(SettleOption.name, func.sum(Settlement.amount))
        .join(Voucher.settlements)
        .join(Settlement.settle_option)
        .filter(Voucher.date >= start_date, Voucher.date <= finish_date)
        .group_by(SettleOption.name)
        .order_by(SettleOption.name)
        .all()
    )
    total = 0
    info = []
    for gt, am in list_:
        total += am
        info.append({"name": gt, "amount": am})
    return info + [{"name": "Total", "amount": total}]


@view_config(
    request_method="GET",
    route_name="v1_sa_tax",
    renderer="json",
    permission=("Tax Analysis", "Sales Analysis"),
)
def get_tax_view(request):
    start_date = request.GET.get("s", None)
    if not start_date:
        start_date = datetime.today().replace(hour=7, minute=0)
    else:
        start_date = datetime.strptime(start_date, "%d-%b-%Y").replace(hour=7, minute=0)

    finish_date = request.GET.get("f", None)
    if not finish_date:
        finish_date = datetime.today().replace(hour=7, minute=0) + timedelta(days=1)
    else:
        finish_date = datetime.strptime(finish_date, "%d-%b-%Y").replace(
            hour=7, minute=0
        ) + timedelta(days=1)

    if (
        datetime.date().today() - start_date.date()
    ).days > 5 and "Accounts Audit" not in request.effective_principals:
        raise ValidationError("Accounts Audit")

    return {
        "startDate": start_date.date().strftime("%d-%b-%Y"),
        "finishDate": (finish_date - timedelta(days=1)).date().strftime("%d-%b-%Y"),
        "amounts": get_settlements(start_date, finish_date, request.dbsession),
    }


def get_tax(start_date, finish_date, dbsession):
    amounts = (
        dbsession.query(
            Tax.name,
            Inventory.tax_rate,
            func.coalesce(func.sum(Inventory.net), 0),
            func.coalesce(func.sum(Inventory.tax_amount), 0),
        )
        .join(Voucher.kots)
        .join(Kot.inventories)
        .join(Inventory.tax)
        .filter(
            Voucher.date >= start_date,
            Voucher.date <= finish_date,
            Voucher.voucher_type == VoucherType.REGULAR_BILL.value,
        )
        .group_by(Tax.name, Inventory.tax_rate)
        .order_by(Tax.name, Inventory.tax_rate)
        .all()
    )
    return [
        {
            "name": "{0} - {1:.2%}".format(i[0], i[1]),
            "taxRate": i[1],
            "netSale": i[2],
            "amount": i[3],
        }
        for i in amounts
    ]
