import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';
import { CashierCheckoutDataSource } from './cashier-checkout-datasource';
import { CashierCheckout } from './cashier-checkout';
import { ToCsvService } from '../shared/to-csv.service';
import { User } from '../core/user';

@Component({
  selector: 'app-cashier-checkout',
  templateUrl: './cashier-checkout.component.html',
  styleUrls: ['./cashier-checkout.component.css']
})
export class CashierCheckoutComponent implements OnInit {
  dataSource: CashierCheckoutDataSource;
  form: FormGroup;
  activeCashiers: User[];
  info: CashierCheckout;

  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['name', 'amount'];


  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private fb: FormBuilder,
    private toCsv: ToCsvService
  ) {
    this.createForm();

  }

  ngOnInit() {
    this.route.data
      .subscribe((data: { cashiers: User[], info: CashierCheckout }) => {
        this.activeCashiers = data.cashiers;
        this.info = data.info;
        this.form.setValue({
          cashier: this.info.user.id,
          startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
          finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate()
        });
        this.dataSource = new CashierCheckoutDataSource(this.info.amounts);
      });
  }

  show() {
    const info = this.getInfo();
    const url = ['checkout'];
    if (!!info.user.id) {
      url.push(info.user.id);
    }
    this.router.navigate(url, {
      queryParams: {
        startDate: info.startDate,
        finishDate: info.finishDate
      }
    });
  }

  createForm() {
    this.form = this.fb.group({
      startDate: '',
      finishDate: '',
      cashier: ''
    });
  }

  getInfo(): CashierCheckout {
    const formModel = this.form.value;

    return {
      user: new User({id: formModel.cashier}),
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY')
    };
  }

  exportCsv() {
    const headers = {
      Date: 'date',
      Name: 'name',
      Type: 'type',
      Narration: 'narration',
      Debit: 'debit',
      Credit: 'credit',
      Running: 'running',
      Posted: 'posted'
    };
    const csvData = new Blob([this.toCsv.toCsv(headers, this.dataSource.data)], {type: 'text/csv;charset=utf-8;'});
    const link = document.createElement('a');
    link.href = window.URL.createObjectURL(csvData);
    link.setAttribute('download', 'cashier-checkout.csv');
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}
