import { Component, OnInit, ViewChild } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { CdkDragDrop, moveItemInArray } from '@angular/cdk/drag-drop';
import { ProductListDataSource } from './product-list-datasource';
import { MatTable } from '@angular/material';
import { Product } from '../../core/product';
import { ToCsvService } from '../../shared/to-csv.service';
import { ToasterService } from '../../core/toaster.service';
import { ProductService } from '../product.service';
import { MenuCategory } from '../../core/menu-category';
import { BehaviorSubject } from 'rxjs';

@Component({
  selector: 'app-product-list',
  templateUrl: './product-list.component.html',
  styleUrls: ['./product-list.component.css']
})
export class ProductListComponent implements OnInit {
  @ViewChild('table', { static: true }) table: MatTable<Product>;
  dataSource: ProductListDataSource;
  filter: BehaviorSubject<string>;
  form: FormGroup;
  list: Product[];
  data: BehaviorSubject<Product[]>;
  menuCategories: MenuCategory[];
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns: string[] = ['name', 'price', 'menuCategory', 'saleCategory', 'info', 'quantity'];

  constructor(
    private route: ActivatedRoute,
    private fb: FormBuilder,
    private router: Router,
    private toaster: ToasterService,
    private toCsv: ToCsvService,
    private ser: ProductService
  ) {
    this.form = this.fb.group({
      menuCategory: ''
    });
    this.filter = new BehaviorSubject('');
    this.data = new BehaviorSubject([]);
    this.data.subscribe((data: Product[]) => {
      this.list = data;
    });
  }

  filterOn(val: any) {
    this.filter.next(val.value);
  }

  ngOnInit() {
    this.route.data
      .subscribe((data: { list: Product[], menuCategories: MenuCategory[] }) => {
        this.data.next(data.list);
        this.menuCategories = data.menuCategories;
      });
    this.dataSource = new ProductListDataSource(this.filter, this.data);
  }

  updateSortOrder() {
    this.ser.updateSortOrder(this.dataSource.viewData)
      .subscribe(
        (result) => {
          this.toaster.show('Success', '');
          this.router.navigateByUrl('/products');
        },
        (error) => {
          this.toaster.show('Danger', error.error);
        }
      );
  }

  dropTable(event: CdkDragDrop<Product[]>) {
    const prevIndex = this.list.indexOf(event.item.data);
    moveItemInArray(this.list, prevIndex, event.currentIndex);
    this.data.next(this.list);
  }

   exportCsv() {
    const headers = {
      Code: 'code',
      Name: 'name',
      Units: 'units',
      Price: 'price',
      MenuCategory: 'menuCategory',
      SaleCategory: 'saleCategory'
    };

    const csvData = new Blob([this.toCsv.toCsv(headers, this.dataSource.viewData)], {type: 'text/csv;charset=utf-8;'});
    const link = document.createElement('a');
    link.href = window.URL.createObjectURL(csvData);
    link.setAttribute('download', 'products.csv');
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}
