import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Product } from '../core/product';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};

const url = '/v1/products';
const serviceName = 'ProductService';

@Injectable({providedIn: 'root'})
export class ProductService {

  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(id: string): Observable<Product> {
    const getUrl: string = (id === null) ? `${url}/new` : `${url}/${id}`;
    return <Observable<Product>>this.http.get<Product>(getUrl)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`))
      );
  }

  list(): Observable<Product[]> {
    const options = {params: new HttpParams().set('l', '')};
    return <Observable<Product[]>>this.http.get<Product[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'list'))
      );
  }

  listIsActiveOfCategory(id: string): Observable<Product[]> {
    const options = {params: new HttpParams().set('mc', id).set('a', '')};
    return <Observable<Product[]>>this.http.get<Product[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'listIsActiveOfCategory'))
      );
  }

  save(product: Product): Observable<Product> {
    return <Observable<Product>>this.http.post<Product>(`${url}/new`, product, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'save'))
      );
  }

  update(product: Product): Observable<Product> {
    return <Observable<Product>>this.http.put<Product>(`${url}/${product.id}`, product, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'update'))
      );
  }

  updateSortOrder(list: Product[]): Observable<boolean> {
    return <Observable<boolean>>this.http.post<Product[]>(url, list, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'updateSortOrder'))
      );
  }

  saveOrUpdate(product: Product): Observable<Product> {
    if (!product.id) {
      return this.save(product);
    } else {
      return this.update(product);
    }
  }

  delete(id: string): Observable<Product> {
    return <Observable<Product>>this.http.delete<Product>(`${url}/${id}`, httpOptions)
      .pipe(
        catchError(this.log.handleError(serviceName, 'delete'))
      );
  }

  autocomplete(term: string): Observable<Product[]> {
    const options = {params: new HttpParams().set('t', term)};
    return <Observable<Product[]>>this.http.get<Product[]>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'autocomplete'))
      );
  }

  balance(id: string, date: string): Observable<number> {
    const options = {params: new HttpParams().set('b', 'true').set('d', date)};
    return <Observable<number>>this.http.get<number>(`${url}/${id}`, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'balance'))
      );
  }
}
