import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { ErrorLoggerService } from '../../core/error-logger.service';
import { Bill, PrintType } from './bill';
import { Table } from '../../core/table';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};

const url = '/v1/vouchers';
const urlMoveTable = '/v1/move-table';
const urlMoveKot = '/v1/move-kot';
const serviceName = 'VoucherService';

@Injectable({providedIn: 'root'})
export class VoucherService {

  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(id: string): Observable<Bill> {
    return <Observable<Bill>>this.http.get<Bill>(`${url}/${id}`)
      .pipe(
        catchError(this.log.handleError(serviceName, `get id=${id}`))
      );
  }

  getFromTable(tableId: string, voucherId: string, guestId: string): Observable<Bill> {
    let params = new HttpParams();
    if (tableId !== null) {
      params = params.set('t', tableId);
    }
    if (voucherId !== null) {
      params = params.set('v', voucherId);
    }
    if (guestId !== null) {
      params = params.set('g', guestId);
    }

    return <Observable<Bill>>this.http.get<Bill>(`${url}/new`, {params: params})
      .pipe(
        catchError(this.log.handleError(
          serviceName,
          `getFromTable tableId=${tableId} voucherId=${voucherId} guestId=${guestId}`
        ))
      );
  }

  save(voucher: Bill, printType: PrintType, guest_book_id: string, updateTable: boolean): Observable<boolean> {
  const options = {params: new HttpParams().set('p', printType).set('u', updateTable.toString())};
  if (guest_book_id !== null) {
    options.params = options.params.set('g', guest_book_id);
  }
  return <Observable<boolean>>this.http.post<boolean>(`${url}/new`, voucher, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'save'))
      );
  }

  update(voucher: Bill, printType: PrintType, guest_book_id: string, updateTable: boolean): Observable<boolean> {
  const options = {params: new HttpParams().set('p', printType).set('u', updateTable.toString())};
  if (guest_book_id !== null) {
    options.params = options.params.set('g', guest_book_id);
  }
    return <Observable<boolean>>this.http.put<boolean>(`${url}/${voucher.id}`, voucher, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'update'))
      );
  }

  saveOrUpdate(voucher: Bill, printType: PrintType, guest_book_id: string, updateTable: boolean): Observable<boolean> {
    if (!voucher.id) {
      return this.save(voucher, printType, guest_book_id, updateTable);
    } else {
      return this.update(voucher, printType, guest_book_id, updateTable);
    }
  }

  receivePayment(id: string, amounts: { id: string; name: string; amount: number }[], updateTable: boolean): Observable<boolean> {
    const options = {params: new HttpParams().set('receive-payment', '').set('u', updateTable.toString())};
    return <Observable<boolean>>this.http.post<boolean>(
      `${url}/${id}`, amounts, options
    ).pipe(
      catchError(this.log.handleError(serviceName, 'receivePayment'))
    );
  }

  voidBill(id: string, reason: string, updateTable: boolean): Observable<boolean> {
    const options = {params: new HttpParams().set('void-bill', '').set('u', updateTable.toString())};
    return <Observable<boolean>>this.http.post<boolean>(
      `${url}/${id}`, {reason: reason}, options
    ).pipe(
      catchError(this.log.handleError(serviceName, 'voidBill'))
    );
  }

  moveTable(id: string, table: Table): Observable<boolean> {
    const options = {params: new HttpParams().set('move', '')};
    return <Observable<boolean>>this.http.post<boolean>(urlMoveTable, {
      voucher: {id: id},
      table: {id: table.id}
    }, options).pipe(
      catchError(this.log.handleError(serviceName, 'moveTable'))
    );
  }

  mergeTable(id: string, table: Table): Observable<boolean> {
    const options = {params: new HttpParams().set('merge', '')};
    return <Observable<boolean>>this.http.post<boolean>(urlMoveTable, {
      voucher: {id: id},
      table: {id: table.id},
      newVoucher: {id: table.voucherId}
    }, options).pipe(
      catchError(this.log.handleError(serviceName, 'mergeTable'))
    );
  }

  moveKotToNewTable(id: string, kotId: string, table: Table): Observable<boolean> {
    const options = {params: new HttpParams().set('move', '')};
    return <Observable<boolean>>this.http.post<boolean>(urlMoveKot, {
      voucher: {id: id},
      kot: {id: kotId},
      table: {id: table.id}
    }, options).pipe(
      catchError(this.log.handleError(serviceName, 'moveKotToNewTable'))
    );
  }

  mergeKotWithOldBill(id: string, kotId: string, table: Table): Observable<boolean> {
    const options = {params: new HttpParams().set('merge', '')};
    return <Observable<boolean>>this.http.post<boolean>(urlMoveKot, {
      voucher: {id: id},
      kot: {id: kotId},
      table: {id: table.id},
      newVoucher: {id: table.voucherId}
    }, options).pipe(
      catchError(this.log.handleError(serviceName, 'mergeKotWithOldBill'))
    );
  }

  splitBill(id: string, inventoriesToMove: string[], table: Table) {
    const options = {params: new HttpParams().set('split-bill', '').set('u', 'true')};
    return <Observable<boolean>>this.http.post<boolean>(`${url}/${id}`, {
      voucher: {id: id},
      inventories: inventoriesToMove,
      table: {id: table.id}
    }, options).pipe(
      catchError(this.log.handleError(serviceName, 'splitBill'))
    );
  }
}
